!================================================================================================
! This is the "toy" chemistry module.
!================================================================================================

module chemistry

  implicit none
  private
  save

  public :: initial_value        ! initialize x and x2
  public :: tendency             ! interface to tendency computation

  integer, parameter :: r8 = selected_real_kind (12)

  real(r8), parameter :: xt_constant = 4.e-6_r8
  real(r8), parameter :: pi = 3.1415926535897932384626433832795028841971693993751058209749445923078164_r8
  real(r8), parameter :: half_pi = pi*0.5_r8
  real(r8), parameter :: degrees_to_radians = pi/180.0_r8
  real(r8), parameter :: k1_lat_center =   20.d0*degrees_to_radians
  real(r8), parameter :: k1_lon_center =  300.d0*degrees_to_radians

contains

  !===============================================================================
  !  Solar photolysis rate and recombination rate
  !===============================================================================

  subroutine k_vals( lat, lon, k1, k2 )

    !-----------------------------------------------------------------------
    ! Arguments:
    !-----------------------------------------------------------------------
    real(r8), intent(in)    :: lat, lon  ! latitude and longitude, radians
    real(r8), intent(out)   :: k1, k2    ! reaction rates

    k1 = 1.0_r8*max(0.d0,sin(lat)*sin(k1_lat_center) + cos(lat)*cos(k1_lat_center)*cos(lon-k1_lon_center))
    k2 = 1._r8

    return

  end subroutine k_vals

  !===============================================================================
  !  Tendencies of x and x2
  !===============================================================================

  subroutine tendency( lat, lon, x, x2, dt, x_f, x2_f )

    !-----------------------------------------------------------------------
    ! Arguments:
    !-----------------------------------------------------------------------

    real(r8), intent(in)    :: lat, lon  ! latitude and longitude, degrees
    real(r8), intent(in)    :: x, x2     ! molar mixing ratio of x and x2
    real(r8), intent(in)    :: dt        ! size of physics time step

    real(r8), intent(out)   :: x_f, x2_f  ! time rate of change of x and x2

    !-----------------------------------------------------------------------
    ! Local variables
    !-----------------------------------------------------------------------

    real(r8) :: r, det, expdt, el ! useful algebraic quantities used in the computation
    real(r8) :: k1, k2            ! reaction rates
    real(r8) :: xt                ! quantity that should be conserved

    call k_vals( lat*degrees_to_radians, lon*degrees_to_radians, k1, k2 )

    r = k1 / (4._r8*k2)
    xt = x + 2._r8* x2

    det = sqrt( r*r + 2._r8*r*xt )
    expdt = exp( -4._r8*k2*det*dt )

    if ( abs(det * k2 * dt) .gt. 1e-16 ) then
       el = (1._r8 - expdt) /det /dt
    else
       el = 4._r8*k2
    endif

    x_f  = -el * (x - det + r)*(x + det + r) / (1._r8 + expdt + dt*el*(x + r))
    x2_f = -x_f / 2._r8

    return

  end subroutine tendency

  !===============================================================================
  !  Compute initial values
  !===============================================================================

  subroutine initial_value( lat, lon, x, x2 )

    !-----------------------------------------------------------------------
    ! Arguments:
    !-----------------------------------------------------------------------

    real(r8), intent(in)  :: lat, lon  ! latitude and longitude, degrees
    real(r8), intent(out) :: x, x2   ! molar mixing ratio of x and x2

    !-----------------------------------------------------------------------
    ! Local variables
    !-----------------------------------------------------------------------

    real(r8) :: r, det  ! useful algebraic forms
    real(r8) :: k1, k2  ! reaction rates

    call k_vals( lat*degrees_to_radians, lon*degrees_to_radians, k1, k2 )

    r = k1 / (4._r8*k2)
    det = sqrt(r*r + 2._r8*xt_constant*r)

    x  = (det-r)
    x2 = xt_constant/2._r8 - (det-r)/2._r8

    return

  end subroutine initial_value

end module chemistry





!===============================================================================
!  Test program
!===============================================================================

program test

use chemistry, only: initial_value, tendency

    integer, parameter :: r8 = selected_real_kind (12)
    real(r8) :: x, x2, xf, x2f, lat, lon, dt
 
    lat = -45.00  ! latitude
    lon =   8.65  ! longitude  (near terminator)
    dt = 1800.00  ! seconds

    call initial_value(lat, lon, x, x2)
    print *, " "
    print *, " Initial Values"
    print *, " Latitude: ",lat," Longitude: ",lon
    print *, " x: ",x," x2: ",x2

    call tendency(lat, lon, x, x2, dt, xf, x2f)
    print *, " "
    print *, " Tendency should be zero for the initial values"
    print *, " x rate ",xf," x2 rate: ",x2f
    print *, " "

end
