;***********************************************
;  Plot contours of Xt with overlays of terminator and wind vectors
;
;  For Xt, contours are as follows
;     (/3.e-6,  3.7e-6, 3.9e-6, 3.97e-6, 3.99e-6, 4.01e-6, 4.03e-6, 4.1e-6, 4.3e-6, 5e-6 /)
;
;  For X,2*X2, contours are as follows
;     (/-1e-7, -1e-8, 1e-8, 1e-7, 1e-6, 2e-6, 3e-6, 3.9e-6, 3.99e-6, 4.01e-6, 4.1e-6/)
;
;  Plotting routine follows
;***********************************************

;***********************************************
;  Include a colleciton of NCL libraries
;***********************************************

  load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
  load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
  load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"

;***********************************************
;  Plotting code
;***********************************************

  begin

     plotfilename = "SE_interpolated"

     filename = "xtm.nc"
     afile = addfile(filename,"r")

     ; import data from file
     xti = afile->Xt
     ui = afile->U
     vi = afile->V

     lats = afile->lat
     lons = afile->lon

     ; extract a particular level of data
     trac_lev = 3   
     xt = xti(0,trac_lev,:,:)  
     u  =   ui(0,trac_lev,:,:)  
     v  =   vi(0,trac_lev,:,:)  

     delete(xt@units)      ; don't show units on plot
     delete(xt@long_name)

;************************************************
;  Open plot file as "png"
;************************************************

     wks = gsn_open_wks("png",plotfilename)


;************************************************
;  Define a color map.
;************************************************

     gsn_define_colormap(wks,"wh-bl-gr-ye-re")

;************************************************
;  To make background color to white
;************************************************

     setvalues wks
       "wkBackgroundColor" : (/ 1.0,1.0,1.0 /)
     end setvalues

;************************************************
;  Define terminator (boundary of photolysis region)
;************************************************

     pi = 4.0*atan(1.0)
     rtd = 180.0/pi  ; radians to degrees
     k1_lat_center =  pi* 20.0/180.0
     k1_lon_center =  pi*300.0/180.0
     longs = fspan(-pi,pi,361)
     latit = rtd*atan( -cos(longs-k1_lon_center)/tan(k1_lat_center) )
     latit!0 = "longs"
     longs = longs*rtd

;************************************************
;  Characterize and plot terminator 
;************************************************

     resxy = True                                 ; set up a second resource list
     resxy@gsnDraw = False                        ; do not draw the plot
     resxy@gsnFrame = False                       ; do not advance the frame
     resxy@xyLineThicknesses = 3.0
     resxy@xyLineColors = "black"

     term_plot  = gsn_csm_xy(wks, longs, latit, resxy )          ; draw second plot

;************************************************
;  Resource for vector and contour plot
;************************************************

     rescon = True

;************************************************
;  Remove map, but add grid to plot
;************************************************

     rescon@mpFillColors = (/-1,-1,-1,-1/)            ; Do not plot land ocean ice 
     rescon@mpGridAndLimbOn = True
     rescon@mpGridSpacingF = 90.0
     rescon@mpGridLineDashPattern = 2
     rescon@mpLimbLineColor = 1
     rescon@mpGridLineColor = 0
     rescon@mpGreatCircleLinesOn = True
     rescon@mpPerimLineThicknessF   = 1.
     rescon@mpCenterLonF =  0.0                       ; Rotate the projection.

;************************************************
;  Set up contour characteristics
;************************************************

     rescon@gsnScalarContour    = True                ; contours desired
     rescon@gsnSpreadColors     = True                ; use full color table
     rescon@gsnSpreadColorStart = 10
     rescon@gsnSpreadColorEnd   = -4
   
     rescon@cnFillOn             = True               ; color plot desired
     rescon@cnLinesOn            = False              ; turn off contour lines
     rescon@cnLineLabelsOn       = False              ; turn off contour line labels
     rescon@cnLevelSelectionMode = "ExplicitLevels"   ; Define your own
     rescon@cnLineLabelsOn       = False              ; No contour labels
     rescon@cnInfoLabelPerimOn   = False
     rescon@cnLevelFlag          = True

;************************************************
;  Contour intervals and labels
;************************************************

     rescon@cnLevels = (/3.e-6,  3.7e-6, 3.9e-6, 3.97e-6, 3.99e-6, 4.01e-6, 4.03e-6, 4.1e-6, 4.3e-6, 5e-6 /)
     rescon@lbLabelStrings  = sprintf("%4.2fe-6",1e6*rescon@cnLevels)   ; Format the labels

;************************************************
;  Contour bar key
;************************************************

     rescon@lbLabelBarOn           = True 
     rescon@lbTitleFontThicknessF  = .08
     rescon@lbAutoManage           = False
     rescon@lbLabelFontHeightF     = .0175
     rescon@lbOrientation          =  "Vertical"  
     rescon@lbPerimOn              =  False

     rescon@pmLabelBarDisplayMode  = "Always"
     rescon@pmLabelBarSide         = "Right" 
     rescon@pmTickMarkDisplayMode  = "Always"

;************************************************
;  Plot curly vectors
;************************************************

     rescon@vcRefMagnitudeF         = 80.0              ; define vector ref mag (80 m/s)
     rescon@vcRefLengthF            = 0.045             ; define length of vec ref
     rescon@vcMinDistanceF          = 0.045             ; spacing between vectors
     rescon@vcRefAnnoOn             = False             ; remove vector key
     rescon@vcGlyphStyle            = "CurlyVector"     ; turn on curley vectors
     rescon@vcLineArrowColor        = "white"           ; change vector color
     rescon@vcLineArrowThicknessF   = 2.0               ; change vector thickness
     rescon@vcVectorDrawOrder       = "PostDraw"        ; draw vectors last

;************************************************
;  Start Plotting
;************************************************

     plot = new(1,graphic)

     rescon@gsnDraw = False                        ; do not draw the plot
     rescon@gsnFrame = False                       ; do not advance the frame

;************************************************
;  Plot xt, and overlay terminator
;************************************************

     ; Draw contours over a map.
     plot = gsn_csm_vector_scalar_map_ce(wks,u(:,:),v(:,:),xt(:,:),rescon) 

     ; now overlay terminator plot into "map"
     overlay(plot,term_plot)           

;************************************************
;  Label Xt on the left
;************************************************

     ; create vertical text
     txres                = True
     txres@txFontHeightF  = 0.03
     txres@txAngleF       = 90
     txid = gsn_create_text(wks, "Xt", txres)

     ; Attach text to plot on left
     amres                = True
     amres@amParallelPosF = -0.65                    ; left = -, right = +
     amres@amJust         = "CenterLeft" 
     annoid = gsn_add_annotation(plot, txid, amres)  

;************************************************
;  Place plot on panel
;************************************************

     gsn_panel(wks, plot, (/1,1/),False)

  end
